import os
import uuid

import numpy as np
from PIL import Image
from io import BytesIO


def binarizing(img, threshold=120):
    """传入image对象进行灰度、二值处理"""
    # img = Image.frombytes("I;16B", (30, 70), img)

    img = Image.open(BytesIO(img))
    img = img.convert("L")  # 转灰度
    pixdata = img.load()
    w, h = img.size
    # 遍历所有像素，大于阈值的为黑色
    for y in range(h):
        for x in range(w):
            if pixdata[x, y] < threshold:
                pixdata[x, y] = 0
            else:
                pixdata[x, y] = 255
    return img


def split(im, code, file=None):
    split_lines = [5, 21, 36, 51, 66]

    y_min = 1
    y_max = 27

    ims = []
    c = 0
    ret = []
    for x_min, x_max in zip(split_lines[:-1], split_lines[1:]):
        dir = "dig/" + code[c] + "/"
        # if not os.path.exists(dir):
        #     os.mkdir(dir)
        name = dir + code[c] + '_' + str(uuid.uuid4()) + '.jpeg'

        crop = im.crop([x_min, y_min, x_max, y_max])
        image = Image.new('RGB', (40, 40), (255, 255, 255))
        image = image.convert('L')
        image.paste(crop, (int((40 - crop.width) / 2), 7, int((40 - crop.width) / 2) + int(crop.width), 33))
        image.thumbnail((28, 28), Image.ANTIALIAS)
        ret.append(np.array(image, dtype=np.float32))
        if file:
            image.save(name)
            file.write("{},{}\n".format(name, code[c]))
            print(name, code[c])
        c = c + 1
    return ret
    # for i in range(0, len(code)):
    #     file_name = "{}.jpeg".format(i)
    #     plt.subplot(8, 3, i)
    #     im = Image.open(file_name).convert("1")
    #     # im=img.filter(ImageFilter.MedianFilter(size=3))
    #     plt.imshow(im)
    #     # 显示截取的图像！
    # plt.show()


def depoint(img):
    """传入二值化后的图片进行降噪"""
    pixdata = img.load()
    w, h = img.size
    for y in range(1, h - 1):
        for x in range(1, w - 1):
            count = 0
            if pixdata[x, y - 1] > 245:  # 上
                count = count + 1
            if pixdata[x, y + 1] > 245:  # 下
                count = count + 1
            if pixdata[x - 1, y] > 245:  # 左
                count = count + 1
            if pixdata[x + 1, y] > 245:  # 右
                count = count + 1
            if pixdata[x - 1, y - 1] > 245:  # 左上
                count = count + 1
            if pixdata[x - 1, y + 1] > 245:  # 左下
                count = count + 1
            if pixdata[x + 1, y - 1] > 245:  # 右上
                count = count + 1
            if pixdata[x + 1, y + 1] > 245:  # 右下
                count = count + 1
            if count > 4:
                pixdata[x, y] = 255
    return img


def pre_to_mem(img, code="aaaa"):
    im = binarizing(img)
    ims = split(im, code)
    if ims:
        pix = np.array(ims, dtype=np.float32)
        pix = pix * np.array([1.0 / 255])
        pix = np.reshape(pix, (4, 28 * 28))
        pix = pix.astype(np.float32)
        return pix
    else:
        return None


if __name__ == "__main__":
    i = 0
    file = open("test_labels.txt", "w+")
    for line in open("img_label.txt"):
        i = i + 1
        if i > 2000:
            break
        img, code = line.strip().split(",")
        try:
            im = binarizing(img)
        except:
            continue
        try:
            split(im, code, file)
        except Exception as e:
            print(e)
